//Fragment Shader
#version 330

in vec3 eyePosition, worldNormal;
in vec2 fUVs;
in vec3 PointLightPosition1, DirectionalLightDirection1;
in vec4 DirLightSpacePos;

layout(location = 0) out vec4 FragColor;
layout(location = 1) out vec4 AddGlow;

uniform sampler2D uColorTexture;
uniform sampler2DShadow uPointLightXP;
uniform vec4 material;					//vec4 in the form (ambient, point, directional, glossyness); so far it was (1, 1, 1, 3)
uniform vec3 dirLightColor;
uniform vec3 AmbientLightColor;
uniform bool uAddGlow = false;
uniform bool uBlackAWhite = false;

float DirLightCalcShadowFactor(vec4 LightSpacePos)
{
	vec3 pos = LightSpacePos.xyz / LightSpacePos.w;
	if (pos.x < 0 || pos.x > 1 || pos.y < 0 || pos.y > 1 || pos.z < 0 || pos.z > 1)
		return 1.0f;
	return textureProj(uPointLightXP, LightSpacePos);

}
 
void main()
{
	
	float specularConst = material[3];
	vec4 uvColor = texture(uColorTexture, fUVs);

	vec3 PointLightColor1 = vec3(0.7f);

	vec3 cameraVec = normalize(eyePosition);
	vec3 worldPointLightDir1 = PointLightPosition1 - eyePosition;
	vec3 PointLightDirection1 = normalize(worldPointLightDir1);

	float SpecularCosPoint1 = clamp(dot(cameraVec, normalize(reflect(PointLightDirection1, worldNormal))), 0, 1);
	float SpecularCosDirection1 = clamp(dot(cameraVec, normalize(reflect(-DirectionalLightDirection1, worldNormal))), 0, 1);
	//SpecularCosPoint1 = 0;
	//SpecularCosDirection1 = 0;


	float cosThetaPoint1 = clamp(dot(worldNormal, PointLightDirection1), 0, 1);
	float cosThetaDirection1 = clamp(dot(worldNormal, -DirectionalLightDirection1), 0, 1);
	//cosThetaPoint1 = 0;
	//cosThetaDirection1 = 0;

	float squaredist1 = worldPointLightDir1.x * worldPointLightDir1.x + worldPointLightDir1.y * worldPointLightDir1.y + worldPointLightDir1.z * worldPointLightDir1.z;


	FragColor = vec4(uvColor.rgb * (AmbientLightColor * material[0]
		+ PointLightColor1 * material[1] * (cosThetaPoint1 + pow(SpecularCosPoint1, specularConst)) / squaredist1
		+ DirLightCalcShadowFactor(DirLightSpacePos) * dirLightColor * material[2] * (cosThetaDirection1 + pow(SpecularCosDirection1, specularConst))
		), uvColor.a);

	if (uAddGlow)
	{
		AddGlow = vec4(0.f, dot(FragColor.rgb, vec3(0.2126f, 0.7152f, 0.0722f)),0.f,.6f);
	}
	else
		AddGlow = vec4(0.f);
	
	if (uBlackAWhite)
		FragColor = vec4(dot(FragColor.rgb, vec3(0.2126f, 0.7152f, 0.0722f)) * vec3(.6f, .3f, .3f), 1.f);

	//FragColor = vec4((normalize(worldNormal)+1)/2, 1.0f);
	//FragColor = vec4(length(DirectionalLightDirection1) - 1.0f, length(DirectionalLightDirection1) - 1.0f, length(DirectionalLightDirection1) - 1.0f, 1.0f);

	//vec3 uvColor = texture(uColorTexture, fUVs).rgb;
	//FragColor = vec4(uvColor, 1.0);
	
	//vec3 uvColor = vec3(1.0 - (1.0 - texture(uPointLightXP, fUVs).x) * 25.0);
	//vec3 uvColor = vec3(textureProj(uPointLightXP, DirLightSpacePos));
	//FragColor = vec4(uvColor, 1.0);
}
